@@ -1,5 +1,5 @@
 """DB Hydra ORM"""
-
+import pymongo
 import pyodbc
 import pandas as pd
 import pymysql as MySQLdb
@@ -8,6 +8,7 @@ import pickle
 
 def read_file(file):
     """Reads txt file -> list"""
+
     with open(file,"r") as f:
         rows = f.readlines()
         for i,row in enumerate(rows):
@@ -24,6 +25,88 @@ def read_connection_details(config_file):
     print(", ".join([db_details['DB_SERVER'],db_details['DB_DATABASE'],db_details['DB_USERNAME']]))
     return(db_details)
 
+
+
+import json
+
+
+
+
+    
+def save_migration(function): #decorator
+    def new_function(instance,*args,**kw):
+        #print(kw,**kw)
+        command=function.__name__
+        if command=="create":
+            migration_dict={"create":{"table_name":instance.name,"columns":instance.columns,"types":instance.types}}
+            print(migration_dict)
+        #TODO: add other methods
+        
+        migrator=instance.db1.migrator
+        migrator.migration_list.append(migration_dict)
+        migrator.migration_list_to_json()
+            
+        
+        function(instance,*args,**kw)
+    return(new_function)
+
+        
+
+class Migrator:
+    def __init__(self,db):
+        self.db=db
+        self.migration_number=1
+        self.migration_list=[]
+
+    def process_migration_dict(self,migration_dict):
+        assert len(migration_dict.keys())==1
+        operation=list(migration_dict.keys())[0]
+        options=migration_dict[operation]
+        if operation=="create":
+            table=MysqlTable(self.db,options["table_name"],options["columns"],options["types"])
+            table.create()
+        elif operation=="drop":
+            table=MysqlTable(self.db,options["table_name"])
+            table.drop()
+        elif operation=="add_column":
+            table=MysqlTable(self.db,options["table_name"])
+            table.add_column(options["column_name"],options["column_type"])
+        elif operation=="modify_column":
+            table=MysqlTable(self.db,options["table_name"])
+            table.modify_column(options["column_name"],options["column_type"])
+        elif operation=="drop_column":
+            table=MysqlTable(self.db,options["table_name"])
+            table.drop_column(options["column_name"])
+            
+    def next_migration(self):
+        self.migration_number+=1
+        self.migration_list=[]
+            
+      
+    def migrate(self,migration_list):
+        for i,migration_dict in enumerate(migration_list):
+            self.process_migration_dict(migration_dict)
+           
+            
+    def migrate_from_json(self,filename):
+        with open(filename,"r") as f:
+            rows=f.readlines()[0].replace("\n","")
+            print(rows)
+        result=json.loads(rows)
+        return(result)
+        
+        
+    def migration_list_to_json(self):
+        result=json.dumps(self.migration_list)
+        
+        with open("migrations//migration-"+str(self.migration_number)+".json","w+") as f:
+            f.write(result)
+     
+
+
+
+
+
 class AbstractDB:
     def __init__(self,config_file="config.ini",db_details=None):
         if db_details is None:    
@@ -54,8 +137,30 @@ class AbstractDB:
     def close_connection(self):
         self.connection.close()
         print("DB connection closed")  
+        
+    def initialize_migrator(self):
+        self.migrator=Migrator(self)
 
+        
 
+class AbstractDBMongo:
+    def __init__(self, config_file="config-mongo.ini", db_details=None):
+        if db_details is None:
+            db_details = read_connection_details(config_file)
+        locally = True
+        if db_details["LOCALLY"] == "False":
+            locally = False
+
+        self.DB_SERVER = db_details["DB_SERVER"]
+        self.DB_DATABASE = db_details["DB_DATABASE"]
+        self.DB_USERNAME = db_details["DB_USERNAME"]
+        self.DB_PASSWORD = db_details["DB_PASSWORD"]
+        if locally:
+            self.connect_locally()
+        else:
+            self.connect_remotely()
+
+        #TODO, execute, close
 class db(AbstractDB):
     def connect_remotely(self):
         
@@ -132,10 +237,10 @@ class Mysqldb(AbstractDB):
        
     def get_all_tables(self):
         sysobjects_table=Table(self, "information_schema.tables",["TABLE_NAME"],["nvarchar(100)"])
-        query="SELECT TABLE_NAME FROM information_schema.tables where TABLE_TYPE='BASE TABLE';"
+        query="SELECT TABLE_NAME,TABLE_TYPE,TABLE_SCHEMA FROM information_schema.tables where TABLE_TYPE='BASE TABLE' AND TABLE_SCHEMA='"+self.DB_DATABASE+"' ;"
         rows=sysobjects_table.select(query)
-        print(rows)
-        return(rows) 
+        tables=[x[0] for x in rows]
+        return(tables) 
        
     def generate_table_dict(self):        
         tables=self.get_all_tables()
@@ -144,7 +249,28 @@ class Mysqldb(AbstractDB):
             table_dict[table]=MysqlTable.init_all_columns(self,table)
         return(table_dict)
     
+   
     
+
+class MongoDb(AbstractDBMongo):
+    def connect_remotely(self):
+        #self.connection = MySQLdb.connect(self.DB_SERVER, self.DB_USERNAME, self.DB_PASSWORD, self.DB_DATABASE)
+        self.connection = pymongo.MongoClient("mongodb+srv://" + self.DB_USERNAME + ":" + self.DB_PASSWORD +"@" + self.DB_SERVER + "/" + self.DB_DATABASE + "?retryWrites=true&w=majority")
+        print(self.connection.list_database_names())
+        self.database = self.connection[self.DB_DATABASE]
+        print(self.database)
+        print("DB connection established")
+
+    def execute(self, query):
+        self.cursor.execute(query)
+        self.connection.commit()
+
+    def get_all_tables(self):
+        return self.database.list_collection_names()
+    def createTable(self, name):
+        return self.database[name]
+
+
 #Tables 
 class AbstractSelectable:
     def __init__(self,db1,name,columns=None):  
@@ -244,7 +370,41 @@ class AbstractTable(AbstractJoinable):
         rows=df.values.tolist()
         self.insert(rows,batch=batch,try_mode=try_mode)
 
+class MongoTable():
+    def __init__(self, db, name):
+        self.name = name
+        self.db = db
+        self.collection = self.db.createTable(name)
+    def drop(self):
+        return self.collection.drop()
+    def insert(self, document):
+        return self.collection.insert_one(document)
 
+    def insertMore(self, documents):
+        return self.collection.insert_many(documents)
+
+    def select(self, query, columns={}):
+
+        if (len(columns) == 0):
+
+            return list(self.collection.find(query))
+        else:
+            return list(self.collection.find(query, columns))
+
+    def select_all(self, query):
+        return list(self.collection.find(query))
+
+    def selectSort(self, query, fieldname, direction, columns={}):
+        if (len(columns) == 0):
+            return list(self.collection.find(query).sort(fieldname, direction))
+        else:
+            return list(self.collection.find(query, columns).sort(fieldname, direction))
+    def delete(self, query={}):
+        return self.collection.delete_many(query)
+    def update(self, query, newvalues):
+        return self.collection.update(query, newvalues)
+    def insertFromDataFrame(self, dataframe):
+        return self.collection.insert_many(dataframe.to_dict)
 class Table(Joinable,AbstractTable):
     def __init__(self,db1,name,columns=None,types=None):
         """Override joinable init"""
@@ -362,8 +522,8 @@ class Table(Joinable,AbstractTable):
                 except IndexError as e:
                     print("Warning: IndexError for foreign key self.columns[fk[parent_column_id]]:",e)
         return(parent_foreign_keys)
-    
-                
+
+             
        
 class MysqlTable(MysqlSelectable,AbstractTable):
     def __init__(self,db1,name,columns=None,types=None):
@@ -391,7 +551,7 @@ class MysqlTable(MysqlSelectable,AbstractTable):
         types=temporary_table.get_all_types()
         return(cls(db1,name,columns,types))
         
-        
+    @save_migration
     def create(self,foreign_keys=None):
         assert len(self.columns)==len(self.types)
         assert self.columns[0]=="id"
@@ -430,6 +590,10 @@ class MysqlTable(MysqlSelectable,AbstractTable):
                     if replace_apostrophes:
                         rows[k][j]=str(rows[k][j]).replace("'","")
                     query+="N'"+str(rows[k][j])+"',"
+                elif "varchar" in self.types[j+1]:
+                    if replace_apostrophes:
+                        rows[k][j]=str(rows[k][j]).replace("'","")
+                    query+="'"+str(rows[k][j])+"',"
                 elif self.types[j+1]=="int":
                     query+=str(rows[k][j])+","
                 elif "date" in self.types[j+1]:
@@ -463,6 +627,21 @@ class MysqlTable(MysqlSelectable,AbstractTable):
         query="ALTER TABLE "+self.name+" ADD FOREIGN KEY ("+parent_id+") REFERENCES "+parent+"(id)"
         print(query)
         self.db1.execute(query)
+        
+        
+    def add_column(self,column_name,column_type):
+        command="ALTER TABLE "+self.name+" ADD COLUMN "+column_name+" "+column_type
+        self.db1.execute(command)
+        
+    def drop_column(self,column_name):
+        command="ALTER TABLE "+self.name+" DROP COLUMN "+column_name
+        self.db1.execute(command)
+
+    def modify_column(self,column_name,new_column_type):
+        command="ALTER TABLE "+self.name+" MODIFY COLUMN "+column_name+" "+new_column_type
+        self.db1.execute(command)
+
+        
 
 
 class XlsxDB:
